#include "os.h"
#include "kheap.h"
#include "task.h"
#include "initrd.h"
#include "syscall.h"

extern ULONG file_data_start;
extern ULONG file_data_end;
extern ULONG prog_start;

extern page_directory_t* current_directory;
extern task_t* current_task;
extern task_t* ready_queue;
extern tss_entry_t tss_entry;

ULONG ramdisk_start = 0x40081000; // In the heap

void moo()
{
  while(TRUE)
  {
      settextcolor(2,0);
      printformat("MOO %d", getpid()); // <-- cow
      settextcolor(15,0);
  }
}

void baa()
{
  while(TRUE)
  {
      settextcolor(4,0);
      printformat("BAA %d", getpid()); // <- sheep
      settextcolor(15,0);
  }
}

void surprise(ULONG a, ULONG b) // <- function with 2 arguments
{
  asm volatile ("push %0" :/* no output registers */: "r" (b));
  asm volatile ("push %0" :/* no output registers */: "r" (a));
  while(TRUE)
  {
      asm volatile ("call _settextcolor");
      printformat("%d", getpid());
      settextcolor(15,0);
  }
}

void test()                    // <- program from outside "loaded" by incbin ...
{
  while(TRUE)
  {
      syscall_puts("5u\n");
      //asm volatile ("call *%0"::"r"( ramdisk_start + (ULONG)&prog_start - (ULONG)&file_data_start ) );
      asm volatile ("call *%0"::"r"( 0x40082372 ) );

  }
}

void test1(ULONG addr, ULONG dummy)
{
  while(TRUE)
  {
      asm volatile ("call *%0"::"r"(addr));
      printformat("%d", getpid());
      settextcolor(15,0);
  }
}

int main()
{
    k_clear_screen();
    printformat("Welcome to PrettyOS 0.08\n");
    // GDT, IDT, ISRS, IRQ, timer, keyboard, paging, interrupts, multitasking
    gdt_install();
    idt_install();
    isrs_install();
    irq_install();
    initODA();
    timer_install();
    keyboard_install();
    sti();
    paging_install();
    tasking_install();

    settextcolor(2,0); printformat("VFS & RAM Disk install\n"); settextcolor(15,0);

    printformat("ramdisk_start: %x file_data_start: %x file_data_end: %x\n", ramdisk_start, &file_data_start, &file_data_end);
    k_memcpy((void*)ramdisk_start, &file_data_start, (ULONG)&file_data_end - (ULONG)&file_data_start); //process.asm
    fs_root = install_initrd(ramdisk_start);
    printformat("fs_root->name: %s fs_root->ptr: %d \n", fs_root->name, fs_root->ptr);

    sleepSeconds(5);

    k_clear_screen();

    settextcolor(2,0); printformat("switch to user mode\n");
    syscall_puts("Hello, user world!\n"); // kernel can either use puts(...) or syscall_puts(...)

    // list the content of files <- data from outside "loaded" by incbin ...
    ULONG i = 0;
    struct dirent* node = 0;
    while( (node = readdir_fs(fs_root, i)) != 0)
    {
        printformat("Found file %s\n",node->name);
        fs_node_t* fsnode = finddir_fs(fs_root, node->name);

        if((fsnode->flags & 0x7) == FS_DIRECTORY)
        {
            printformat("\t(directory)\n");
        }
        else
        {
            printformat("\t contents: \"");
            CHAR buf[256];
            ULONG sz = read_fs(fsnode, 0, fsnode->length, buf);
            ULONG j;
            for(j=0; j<sz; ++j)
            printformat("%c",buf[j]);
            printformat("\"\n");
        }
        ++i;
    }

    ///TEST
    printformat("%x\n",ramdisk_start + (ULONG)&prog_start - (ULONG)&file_data_start);
    ///TEST

    sleepSeconds(10);

    settextcolor(15,0);

    task_t* task2 = create_task (moo,0);           // function moo in kernel
    task_t* task3 = create_task (baa,0);           // function baa in kernel
    task_t* task4 = create_task2(surprise,0,4,2);  // function ... in kernel

    task_t* task5 = create_task (test, 3);
    task_t* task6 = create_task2(test1,0,ramdisk_start+(ULONG)&prog_start-(ULONG)&file_data_start,0);

    k_clear_screen();
    log_task_list();
    sleepSeconds(5);

    pODA->ts_flag = 1; // enable task_switching

    k_clear_screen();
    while(TRUE)
    {
        settextcolor(15,0);
        printformat("%d", getpid());
    }
    return 0;
}
